_compliance_endpoints = [
    [
        'compliance_control_list', 
        '/api/2.0/fo/compliance/control/?action=list', 
        [

            {
                'param_name': 'echo_request'

            },
            {
                'param_name': 'details'
            },
            {
                'param_name': 'ids'
            },
            {
                'param_name': 'id_min'
            },
            {
                'param_name': 'id_max'
            },
            {
                'param_name': 'id_min'
            },
            {
                'param_name': 'updated_after_datetime'
            },
            {
                'param_name': 'created_after_datetime'
            },
            {
                'param_name': 'truncation_limit'
            }           
        ]
    ],
    [
         'compliance_policy_list', 
         '/api/2.0/fo/compliance/policy/?action=list',
         [
             {
                 'param_name': 'echo_request', 
             },
             {
                 'param_name': 'details'
             },
             {
                 'param_name': 'ids'
             },
             {
                 'param_name': 'id_min'
             },
             {
                 'param_name': 'id_max'
             },
             {
                 'param_name': 'updated_after_datetime'
             },
             {
                 'param_name': 'created_after_datetime'
             }           

         ]
    ],
    [
        'compliance_policy_export',
        '/api/2.0/fo/compliance/policy/?action=export',
        [
            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'id',
            },
                        {
                'param_name': 'title',
            },
                        {
                'param_name': 'show_user_controls',
            },
                        {
                'param_name': 'show_appendix',
            }

        ]
    ],
    [
        'compliance_policy_import',
        '/api/2.0/fo/compliance/policy/?action=import',
        [

            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'xml_file',
                'is_required': True
            },
            {
                'param_name': 'title',
                'is_required': True
            },
            {
                'param_name': 'create_user_controls'
            }
        ]
    ],
    [
        'compliance_policy_merge',
        '/api/2.0/fo/compliance/policy/?action=merge',
        [

            {
                'param_name': 'id',
                'is_required': True
            },
            {
                'param_name': 'merge_policy_id',
                'is_required': True
            },
            {
                'param_name': 'replace_cover_page'
            },
            {
                'param_name': 'replace_asset_groups'
            },
            {
                'param_name': 'add_asset_groups'
            },
            {
                'param_name': 'add_new_technologies'
            },
            {
                'param_name': 'add_new_controls'
            },
            {
                'param_name': 'update_section_heading'
            },
            {
                'param_name': 'update_existing_controls'
            },
            {
                'param_name': 'preview_merge'
            }
            
        ]
    ],
    [
        'compliance_policy_add_asset_group_ids',
        '/api/2.0/fo/compliance/policy/?action=add_asset_group_ids',
        [
            {
                'param_name': 'id',
                'is_required': True
            },
            {
                'param_name': 'asset_group_ids',
                'is_required': True
            },
            {
                'param_name': 'evaluate_now'
            },            
        ]
    ],
    [
        'compliance_policy_set_asset_group_ids',
        '/api/2.0/fo/compliance/policy/?action=set_asset_group_ids',
        [
            {
                'param_name': 'id',
                'is_required': True
            },
            {
                'param_name': 'asset_group_ids',
                'is_required': True
            },
            {
                'param_name': 'evaluate_now'
            },            
        ]
    ],
    [
        'compliance_policy_add_asset_groups',
        '/api/2.0/fo/compliance/policy/?action=add_asset_group_ids',
        [
            {
                'param_name': 'id',
                'is_required': True
            },
            {
                'param_name': 'asset_group_ids',
                'is_required': True
            },
            {
                'param_name': 'evaluate_now'
            },            
        ]
    ],
    [
        'compliance_posture_list',
        '/api/2.0/fo/compliance/posture/info/?action=list',
        [
            {
                'param_name': 'policy_id',
                'is_required': True
            },
            {
                'param_name': 'policy_ids'
            },
            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'output_format'
            },
            {
                'param_name': 'details'
            },
            {
                'param_name': 'hide_evidence'
            },
            {
                'param_name': 'include_dp_name'
            },
            {
                'param_name': 'show_remediation_info'
            },
            {
                'param_name': 'cause_of_failure'
            },
            {
                'param_name': 'truncation_limit'
            },
            {
                'param_name': 'ips'
            },
            {
                'param_name': 'host_ids'
            },
            {
                'param_name': 'control_ids'
            },
            {
                'param_name': 'ids'
            },
            {
                'param_name': 'id_min'
            },
            {
                'param_name': 'id_max'
            },
            {
                'param_name': 'status_changes_since'
            },
            {
                'param_name': 'evaluation_date'
            },
            {
                'param_name': 'asset_group_ids'
            },
            {
                'param_name': 'status'
            },
            {
                'param_name': 'criticality_labels'
            },
            {
                'param_name': 'criticality_values'
            },
            {
                'param_name': 'tag_set_by'
            },
            {
                'param_name': 'tag_include_selector'
            },
            {
                'param_name': 'tag_exclude_selector'
            },
            {
                'param_name': 'tag_set_include'
            },
            {
                'param_name': 'tag_set_exclude'
            }
        ]
    ],
    [
        'compliance_exception_list',
        '/api/2.0/fo/compliance/exception/?action=list',
        [
            {
                'param_name': 'exception_number'
            },
            {
                'param_name': 'ip',
            },
            {
                'param_name': 'network_name',
            },
            {
                'param_name': 'status',
            },
            {
                'param_name': 'control_id',
            },
            {
                'param_name': 'control_statement',
            },
            {
                'param_name': 'policy_id',
            },
            {
                'param_name': 'technology_name',
            },
            {
                'param_name': 'assignee_id',
            },
            {
                'param_name': 'created_by',
            },
            {
                'param_name': 'modified_by',
            },
            {
                'param_name': 'details',
            },
            {
                'param_name': 'is_active',
            },
            {
                'param_name': 'created_after_date',
            },
            {
                'param_name': 'updated_after_date',
            },
            {
                'param_name': 'expired_before_date',
            },
            {
                'param_name': 'expired_after_date',
            },
            {
                'param_name': 'exception_numbers',
            },
            {
                'param_name': 'exception_number_min',
            },
            {
                'param_name': 'exception_number_max',
            },
            {
                'param_name': 'truncation_limit',
            }

        ]
    ],
    [
        'compliance_exception_request',
        '//api/2.0/fo/compliance/exception/?action=request',
        [
            {
                'param_name': 'control_id',
                'is_required': True
            },
            {
                'param_name': 'host_id',
                'is_required': True
            },
            {
                'param_name': 'policy_id',
                'is_required': True
            },
            {
                'param_name': 'technology_id',
                'is_required': True
            },
            {
                'param_name': 'instance_string',
            },
            {
                'param_name': 'assignee_id',
                'is_required': True
            },
            {
                'param_name': 'comments',
                'is_required': True
            },
            {
                'param_name': 'reopen_on_evidence_change',
            }

        ]
    ],
    [
        'compliance_exception_update',
        '//api/2.0/fo/compliance/exception/?action=update',
        [
            {
                'param_name': 'exception_numbers',
                'is_required': True
            },
            {
                'param_name': 'comments',
                'is_required': True
            },
            {
                'param_name': 'reassign_to',
            },
            {
                'param_name': 'reopen_on_evidence_change',
            },
            {
                'param_name': 'status',
            },
            {
                'param_name': 'end_date',
            }

        ]
    ],
    [
        'compliance_exception_delete',
        '//api/2.0/fo/compliance/exception/?action=delete',
        [
            {
                'param_name': 'exception_numbers',
                'is_required': True
            }

        ]
    ],
    
]


from ._ep_reports import _reports_endpoints 
from ._ep_assets import _assets_endpoints
from ._ep_compliance import _compliance_endpoints
from ._ep_session import _session_endpoints
from .endpoint import APIAction

api_actions = {}


all_endpoints = [*_reports_endpoints, 
                *_assets_endpoints,
                *_compliance_endpoints,
                *_session_endpoints]

for a in all_endpoints:
    api_actions[a[0]]  = APIAction(a[0],
        api_endpoint=a[1],
        input_params=a[2]
        )




from typing import List


class InputParameter:

    def __init__(self, param_name:str, is_required:bool = False, **kwargs):
        self.param_name = param_name
        self.is_required = is_required
        self.description = kwargs.get('description')

    def __str__(self):
        _ = f'{self.param_name} :: {self.is_required}'
        if (self.description):
            _ = _ + ' :: ' + self.description
        return _

InputParameters = List[InputParameter]

class APIAction:

    def __init__(self, action_name, api_endpoint:str, input_params:InputParameters, http_method = 'POST'):
        self.action_name = action_name
        self.api_endpoint:str = api_endpoint
        self.input_parameters = [InputParameter(**ip) for ip in input_params]
        self.http_method = http_method

        
        
    def add_summary(self, summary:str):
        self.summary:str = summary

    @property
    def action_name(self):
        return self.__action_name

    @action_name.setter
    def action_name(self, action_name):
        if (isinstance(action_name, str)):
            self.__action_name = action_name
        else:
            raise Exception('action_name must be of type str')

    @property
    def input_parameters(self):
        return self.__input_parameters

    @input_parameters.setter
    def input_parameters(self, input_params:InputParameters):
        self.__input_parameters = input_params

    def __str__(self):
        _ = f'API Action Name: {self.action_name}\n'
        _ = _ + f'\tAPI Endpoint: {self.api_endpoint}\n'
        _ = _ + f'\tHTTP Method: {self.http_method}\n'
        _ = _ + '\tInput Parameters: \n\t\t'+"\n\t\t".join([str(ip) for ip in self.input_parameters])
        return _
        
    def get_valid_input_parameters(self):
        return [input_param.param_name for input_param in self.input_parameters]

    def get_required_input_parameters(self):
        return [input_param.param_name for input_param in self.input_parameters if input_param.is_required == True]
    
    def get_optional_input_parameters(self):
        return [input_param.param_name for input_param in self.input_parameters if input_param.is_required == False]



'''
get_report_list()
    uses APIAction get_report_list
    find by lookup of term "api_action_list" in some list
    /api/2.0/fo/report/


'''


_session_endpoints = [
    [
        'login',
        '/api/2.0/fo/session/?action=login',
        [
            {
                'param_name': 'username',
                'is_required': True
            },
            {
                'param_name': 'password',
                'is_required': True
            },
            
        ]

    ]
]


_reports_endpoints = [
    [
        'list_reports', 
        '/api/2.0/fo/report/?action=list', 
        [
            {
                'param_name': 'echo_request'

            },
            {
                'param_name': 'id',
                'description': 'some description'
            },
            {
                'param_name': 'state'
            }
            
        ]
    ],
    [
         'launch_report', 
         '/api/2.0/fo/report/?action=launch',
         [
             {
                 'param_name': 'template_id',
                 'is_required': True
             }
         ]
    ],
    [
        'launch_scorecard',
        '/api/2.0/fo/report/scorecard/?action=launch',
        [
            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'name'
            },
            {
                'param_name': 'report_title'
            },
            {
                'param_name': 'output_format'
            },

        ]
    ],
    [
        'cancel_running_report',
        '/api/2.0/fo/report/?action=cancel',
        [
            {
                'param_name': 'id',
                'is_required': True
            },
            {
                'param_name': 'echo_request'
            },
        ]
    ],
    [
        'download_saved_report',
        '/api/2.0/fo/report/?action=fetch',
        [
            {
                'param_name': 'id',
                'is_required': True
            },
            {
                'param_name': 'echo_request'
            },
        ]
    ]
    
]


_assets_endpoints = [
    [
        'list_ip', 
        '/api/2.0/fo/asset/ip/?action=list', 
        [

            {
                'param_name': 'echo_request'

            },
            {
                'param_name': 'ips',
                'description': '(Optional) Show only certain IP addresses/ranges. One or more IPs/ranges may be specified. Multiple entries are comma separated. A host IP range is specified with a hyphen (for example, 10.10.10.44-10.10.10.90).'
            },
            {
                'param_name': 'network_id'
            },
            {
                'param_name': 'tracking_method'
            }
            
        ]
    ],
    [
         'add_ips', 
         '/api/2.0/fo/asset/ip/?action=add',
         [
             {
                 'param_name': 'action', 
                 'is_required': True
             },
             {
                 'param_name': 'ips',
                 'is_required': True
             }
         ]
    ],
    [
        'update_ips',
        '/api/2.0/fo/asset/ip/?action=update',
        [
            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'show_asset_id',
            },
                        {
                'param_name': 'details',
            },
                        {
                'param_name': 'os_pattern',
            },
                        {
                'param_name': 'truncation_limit',
            },
                        {
                'param_name': 'ips',
            },
                        {
                'param_name': 'show_asset_id',
            },
                        {
                'param_name': 'no_vm_scan_since'
            },

        ]
    ],
    [
        'host_list',
        '/api/2.0/fo/asset/host/?action=list',
        [

            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'show_asset_id'
            },
            {
                'param_name': 'details'
            },
            {
                'param_name': 'os_pattern'
            },
            {
                'param_name': 'truncation_limit',
                'default': '10'
            },
            {
                'param_name': 'ips'
            },
            {
                'param_name': 'ipv6'
            },
            {
                'param_name': 'ag_ids'
            },
            {
                'param_name': 'ag_titles'
            },
            {
                'param_name': 'ids'
            },
            {
                'param_name': 'id_min'
            },
            {
                'param_name': 'id_max'
            },
            {
                'param_name': 'network_ids'
            },
            {
                'param_name': 'compliance_enabled'
            },
            {
                'param_name': 'no_vm_scan_since'
            },
            {
                'param_name': 'no_compliance_scan_since'
            },
            {
                'param_name': 'vm_scan_since'
            },
            {
                'param_name': 'compliance_scan_since'
            },
            {
                'param_name': 'vm_processed_before'
            },
            {
                'param_name': 'vm_processed_after'
            },
            {
                'param_name': 'vm_scan_date_before'
            },
            {
                'param_name': 'vm_scan_date_after'
            },
            {
                'param_name': 'vm_auth_scan_date_before'
            },
            {
                'param_name': 'vm_auth_scan_date_after'
            },
            {
                'param_name': 'scap_scan_since'
            },
            {
                'param_name': 'no_scap_scan_since'
            },

            {
                'param_name': 'use_tags'
            },
            {
                'param_name': 'tag_set_by'
            },
            {
                'param_name': 'tag_include_selector'
            },
            {
                'param_name': 'tag_exclude_selector'
            },
            {
                'param_name': 'tag_set_include'
            },
            {
                'param_name': 'tag_set_exclude'
            },
            {
                'param_name': 'show_tags'
            },
            {
                'param_name': 'host_metadata'
            },
            {
                'param_name': 'host_metadata_fields'
            },
            {
                'param_name': 'show_cloud_tags'
            },
            {
                'param_name': 'cloud_tag_fields'
            },
            
        ]
    ],

    [
        'host_list_detection',
        '/api/2.0/fo/asset/host/vm/detection/?action=list',
        [

            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'show_results'
            },
            {
                'param_name': 'show_reopened_info'
            },
            {
                'param_name': 'arf_kernel_filter'
            },
            {
                'param_name': 'arf_service_filter'
            },
            {
                'param_name': 'arf_config_filter'
            },
            {
                'param_name': 'active_kernels_only'
            },
            {
                'param_name': 'output_format'
            },
            {
                'param_name': 'suppress_duplicated_data_from_csv'
            },
            {
                'param_name': 'truncation_limit'
            },
            {
                'param_name': 'max_days_since_detection_updated'
            },
            {
                'param_name': 'detection_updated_since'
            },
            {
                'param_name': 'detection_updated_before'
            },
            {
                'param_name': 'detection_processed_before'
            },
            {
                'param_name': 'detection_processed_after'
            },
            {
                'param_name': 'detection_last_tested_since'
            },
            {
                'param_name': 'detection_last_tested_since_days'
            },
            {
                'param_name': 'detection_last_tested_before'
            },
            {
                'param_name': 'detection_last_tested_before_days'
            },
            {
                'param_name': 'include_ignored'
            },
            {
                'param_name': 'include_disabled'
            },
            {
                'param_name': 'ids'
            },
            {
                'param_name': 'id_min'
            },
            {
                'param_name': 'id_max'
            },
            {
                'param_name': 'ips'
            },
            {
                'param_name': 'ipv6'
            },
            {
                'param_name': 'ag_ids'
            },
            {
                'param_name': 'ag_titles'
            },
            {
                'param_name': 'network_ids'
            },
            {
                'param_name': 'vm_scan_since'
            },
            {
                'param_name': 'no_vm_scan_since'
            },
            {
                'param_name': 'max_days_since_last_vm_scan'
            },
            {
                'param_name': 'vm_processed_before'
            },
            {
                'param_name': 'vm_processed_after'
            },
            {
                'param_name': 'vm_scan_date_before'
            },
            {
                'param_name': 'vm_scan_date_after'
            },
            {
                'param_name': 'vm_auth_scan_date_before'
            },
            {
                'param_name': 'vm_auth_scan_date_after'
            },
            {
                'param_name': 'status'
            },
            {
                'param_name': 'compliance_enabled'
            },
            {
                'param_name': 'os_pattern'
            },
            {
                'param_name': 'qids'
            },
            {
                'param_name': 'severities'
            },
            {
                'param_name': 'filter_superseded_qids'
            },
            {
                'param_name': 'show_igs'
            },
            {
                'param_name': 'include_search_list_titles'
            },
            {
                'param_name': 'exclude_search_list_titles'
            },
            {
                'param_name': 'include_search_list_ids'
            },
            {
                'param_name': 'exclude_search_list_ids'
            },
            {
                'param_name': 'use_tags'
            },
            {
                'param_name': 'tag_set_by'
            },
            {
                'param_name': 'tag_include_selector'
            },
            {
                'param_name': 'tag_exclude_selector'
            },
            {
                'param_name': 'tag_set_include'
            },
            {
                'param_name': 'tag_set_exclude'
            },
            {
                'param_name': 'show_tags'
            },
            {
                'param_name': 'host_metadata'
            },
            {
                'param_name': 'host_metadata_fields'
            },
            {
                'param_name': 'show_cloud_tags'
            },
            {
                'param_name': 'cloud_tag_fields'
            },
            {
                'param_name': 'LAST_SCAN_DATETIME'
            },
            {
                'param_name': 'LAST_VM_SCANNED_DATE'
            },
            {
                'param_name': 'LAST_VM_SCANNED_DURATION'
            },
            {
                'param_name': 'LAST_VM_AUTH_SCANNED_DATE'
            },
            {
                'param_name': 'LAST_VM_AUTH_SCANNED_DURATION'
            },
            {
                'param_name': 'LAST_PC_SCANNED_DATE'
            },
            {
                'param_name': 'FIRST_FOUND_DATETIME'
            },
            {
                'param_name': 'LAST_FOUND_DATETIME'
            },
            {
                'param_name': 'LAST_TEST_DATETIME'
            },
            {
                'param_name': 'LAST_UPDATE_DATETIME'
            },
            {
                'param_name': 'LAST_FIXED_DATETIME'
            }          

            
        ]
    ],
    [
        'excluded_host_list',
        '/api/2.0/fo/asset/excluded_ip/?action=list',
        [
            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'ips'
            },
            {
                'param_name': 'network_id'
            },
            {
                'param_name': 'ag_ids'
            },
            {
                'param_name': 'ag_titles'
            },
            {
                'param_name': 'use_tags'
            },
            {
                'param_name': 'tag_include_selector'
            },
            {
                'param_name': 'tag_exclude_selector'
            },
            {
                'param_name': 'tag_set_by'
            },
            {
                'param_name': 'tag_set_include'
            },
            {
                'param_name': 'tag_set_exclude'
            }
            
        ]
    ],
    [
        'excluded_host_change_history',
        '/api/2.0/fo/asset/excluded_ip/history/?action=list',
        [
            {
                'param_name': 'echo_request'
            },
            {
                'param_name': 'ips'
            },
            {
                'param_name': 'network_id'
            },
            {
                'param_name': 'id_min'
            },
            {
                'param_name': 'id_max'
            },
            {
                'param_name': 'ids'
            }
            
        ]
    ]
    
    
]


from setuptools import setup, find_packages


with open("README.md", "r") as fh:
    long_description = fh.read()

exec(open('qualysclient/version.py').read())

setup(
    name="qualysclient",
    version=__version__,
    author="Wood Techie",
    author_email="woodtechie1428@gmail.com",
    description="Python SDK for interacting with the Qualys API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/woodtechie1428/qualysclient",
    project_urls={
        "Bug Tracker": "https://github.com/woodtechie1428/qualysclient/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "qualysclient"},
    packages=find_packages(where="qualysclient"),
    python_requires=">=3.6",
    py_modules=["qualysclient"],
    install_requires=['requests']

)
